package micro

import (
	"context"
	"git.nspix.com/golang/micro/registry"
)

type (
	Options struct {
		Zone                   string            //注册域
		Name                   string            //名称
		Version                string            //版本号
		EnableHttp             bool              //启用HTTP功能
		EnableRPC              bool              //启用RPC功能
		EnableInternalListener bool              //启用内置网络监听服务
		registry               registry.Registry //注册仓库
		Server                 Server            //加载的服务
		Port                   int               //绑定端口
		Address                string            //绑定地址
		Context                context.Context
	}

	Option func(o *Options)
)

func WithName(name string, version string) Option {
	return func(o *Options) {
		o.Name = name
		o.Version = version
	}
}

func WithPort(port int) Option {
	return func(o *Options) {
		o.Port = port
	}
}

func WithRegistry(r registry.Registry) Option {
	return func(o *Options) {
		o.registry = r
	}
}

func WithServer(s Server) Option {
	return func(o *Options) {
		o.Server = s
	}
}

func WithoutHttp() Option {
	return func(o *Options) {
		o.EnableHttp = false
	}
}

func WithoutRPC() Option {
	return func(o *Options) {
		o.EnableRPC = false
	}
}

func NewOptions() *Options {
	return &Options{
		Zone:                   "default",
		Version:                "1.0.1",
		EnableHttp:             true,
		EnableRPC:              true,
		EnableInternalListener: true,
		Context:                context.Background(),
		registry:               registry.DefaultRegistry,
	}
}
